package com.learnfiles.gmailinbox.activity;

import android.content.res.TypedArray;
import android.graphics.Color;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.view.ActionMode;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Toast;

import com.learnfiles.gmailinbox.R;
import com.learnfiles.gmailinbox.adapter.MessagesAdapter;
import com.learnfiles.gmailinbox.model.Message;
import com.learnfiles.gmailinbox.network.ApiClient;
import com.learnfiles.gmailinbox.network.ApiInterface;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class MainActivity extends AppCompatActivity implements MessagesAdapter.MessageAdapterListener {


    RecyclerView recycle_view;
    List<Message> messages=new ArrayList<>();
    MessagesAdapter mAdapter;
    SwipeRefreshLayout swipe_refresh_layout;
    ActionMode actionMode;
    ActionModeCallback actionModeCallback;



    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        Toolbar toolbar=(Toolbar)findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);


        swipe_refresh_layout=(SwipeRefreshLayout)findViewById(R.id.swipe_refresh_layout);
        recycle_view=(RecyclerView)findViewById(R.id.recycler_view) ;

        RecyclerView.LayoutManager mLayoutManager=new LinearLayoutManager(getApplicationContext());
        recycle_view.setLayoutManager(mLayoutManager);



        mAdapter=new MessagesAdapter(this,messages,this);


        recycle_view.setAdapter(mAdapter);



        swipe_refresh_layout.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                getInbox();
            }
        });


    swipe_refresh_layout.post(new Runnable() {
        @Override
        public void run() {

            getInbox();

        }
    });


        FloatingActionButton fab=(FloatingActionButton) findViewById(R.id.fab);

        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Snackbar.make(view,"Your code ...",Snackbar.LENGTH_LONG).setAction("Action",null).show();
            }
        });



        actionModeCallback=new ActionModeCallback();

    }




    public void getInbox()
    {


        swipe_refresh_layout.setRefreshing(true);


        ApiInterface apiService = ApiClient.getClient().create(ApiInterface.class);


        Call<List<Message>> call=apiService.getInbox();

        call.enqueue(new Callback<List<Message>>() {
            @Override
            public void onResponse(Call<List<Message>> call, Response<List<Message>> response) {
                messages.clear();


                for(Message message:response.body())
                {
                    message.setColor(getRandomMaterialColor());
                    messages.add(message);
                }

                mAdapter.notifyDataSetChanged();

                swipe_refresh_layout.setRefreshing(false);

            }

            @Override
            public void onFailure(Call<List<Message>> call, Throwable t) {

                Toast.makeText(getApplicationContext(),"Unable to fetch json : "+t.getMessage(),Toast.LENGTH_SHORT).show();
                swipe_refresh_layout.setRefreshing(false);

            }
        });

    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        getMenuInflater().inflate(R.menu.main_menu,menu);

        return true;
    }


    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {

        if (item.getItemId()==R.id.action_search)
        {
            Toast.makeText(getApplicationContext(),"Search ...",Toast.LENGTH_SHORT).show();

            return true;
        }


        return super.onOptionsItemSelected(item);
    }



    public int getRandomMaterialColor()
    {
        int returnColor= Color.GRAY;


        int arrayId=getResources().getIdentifier("mdcolor_400","array",getPackageName());

        if (arrayId!=0)
        {
            TypedArray colors=getResources().obtainTypedArray(arrayId);

            int index=(int)(Math.random()*colors.length());

            returnColor = colors.getColor(index,Color.GRAY);

            colors.recycle();

        }

        return returnColor;


    }

    @Override
    public void onIconClicked(int position) {

            if (actionMode==null)
            {
                actionMode=startSupportActionMode(actionModeCallback);
            }

            toggleSelection(position);
    }

    @Override
    public void onMessageRowClicked(int position) {
        if (mAdapter.getSelectedItemCount()>0)
        {
            if (actionMode==null)
            {
                actionMode=startSupportActionMode(actionModeCallback);
            }

            toggleSelection(position);


        }else {
            Message message = messages.get(position);
            message.setRead(true);
            messages.set(position, message);
            mAdapter.notifyDataSetChanged();
        }
    }

    @Override
    public void onRowLongClicked(int position) {
        if (actionMode==null)
        {
            actionMode=startSupportActionMode(actionModeCallback);
        }

        toggleSelection(position);
    }


    public void toggleSelection(int pos)
    {
        mAdapter.toggleSelection(pos);

        int count=mAdapter.getSelectedItemCount();

        if (count==0)
        {
            actionMode.finish();
        }
        else
        {
            actionMode.setTitle(String.valueOf(count));
            actionMode.invalidate();
        }
    }


    class ActionModeCallback implements ActionMode.Callback{

        @Override
        public boolean onCreateActionMode(ActionMode mode, Menu menu) {
            mode.getMenuInflater().inflate(R.menu.menu_action_mode,menu);

            swipe_refresh_layout.setEnabled(false);

            return true;
        }

        @Override
        public boolean onPrepareActionMode(ActionMode mode, Menu menu) {
            return false;
        }

        @Override
        public boolean onActionItemClicked(ActionMode mode, MenuItem item) {

            if (item.getItemId()==R.id.action_delete)
            {


                mAdapter.resetAnimationIndex();


                List<Integer> selectedItemPositions=mAdapter.getSelectedItems();

                for (int i=selectedItemPositions.size()-1;i>=0;i--)
                {
                    mAdapter.removeData(selectedItemPositions.get(i));
                }

                mAdapter.notifyDataSetChanged();



                mode.finish();

                return true;
            }

            return false;
        }

        @Override
        public void onDestroyActionMode(ActionMode mode) {
            mAdapter.clearSelections();
            swipe_refresh_layout.setEnabled(true);
            actionMode=null;


            recycle_view.post(new Runnable() {
                @Override
                public void run() {
                    mAdapter.resetAnimationIndex();
                }
            });

        }
    }


}
