package com.learnfiles.gmailinbox.adapter;


import android.content.Context;
import android.content.Intent;
import android.content.res.TypedArray;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Typeface;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.util.SparseBooleanArray;
import android.view.HapticFeedbackConstants;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.learnfiles.gmailinbox.R;
import com.learnfiles.gmailinbox.helper.CircleTransform;
import com.learnfiles.gmailinbox.helper.FlipAnimator;
import com.learnfiles.gmailinbox.model.Message;

import java.util.ArrayList;
import java.util.List;

public class MessagesAdapter extends RecyclerView.Adapter<MessagesAdapter.MyViewHolder> {

    Context mContext;
    List<Message> messages;
    SparseBooleanArray selectedItems;
    SparseBooleanArray animationItemsIndex;
     int  currentSelectedIndex=-1;
    boolean reverseAllAnimations=false;
    MessageAdapterListener listener;


    public MessagesAdapter(Context mContext,List<Message> messages,MessageAdapterListener listener)
    {
        this.mContext=mContext;
        this.messages=messages;

        selectedItems=new SparseBooleanArray();
        animationItemsIndex=new SparseBooleanArray();
        this.listener=listener;
    }


    @Override
    public MyViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {

        View itemView= LayoutInflater.from(parent.getContext()).inflate(R.layout.message_list_row,parent,false);

        return new MyViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(MyViewHolder holder, final int position) {

        final Message message=messages.get(position);

        holder.from.setText(message.getFrom());

        holder.subject.setText(message.getSubject());

        holder.message.setText(message.getMessage());

        holder.timestamp.setText(message.getTimestamp());

        holder.iconText.setText(message.getFrom().substring(0,1));


        holder.itemView.setActivated(selectedItems.get(position,false));

        holder.iconContainer.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
               listener.onIconClicked(position);
            }
        });


        holder.messageContainer.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                listener.onMessageRowClicked(position);
            }
        });



        holder.messageContainer.setOnLongClickListener(new View.OnLongClickListener() {
            @Override
            public boolean onLongClick(View view) {
                listener.onRowLongClicked(position);
                view.performHapticFeedback(HapticFeedbackConstants.LONG_PRESS);
                return true;
            }
        });




        if (message.isRead())
        {
            holder.from.setTypeface(null, Typeface.NORMAL);
            holder.subject.setTypeface(null, Typeface.NORMAL);

            holder.from.setTextColor(ContextCompat.getColor(mContext,R.color.subject));
            holder.subject.setTextColor(ContextCompat.getColor(mContext,R.color.message));


        }else
        {

            holder.from.setTypeface(null, Typeface.BOLD);
            holder.subject.setTypeface(null, Typeface.BOLD);

            holder.from.setTextColor(ContextCompat.getColor(mContext,R.color.from));
            holder.subject.setTextColor(ContextCompat.getColor(mContext,R.color.subject));


        }





        holder.iconImp.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                message.setImportant(!message.isImportant());
                notifyDataSetChanged();

            }
        });



        if (message.isImportant())
        {
            holder.iconImp.setImageDrawable(ContextCompat.getDrawable(mContext,R.drawable.ic_star_black_24dp));
            holder.iconImp.setColorFilter(ContextCompat.getColor(mContext,R.color.icon_tint_selected));

        }else
        {
            holder.iconImp.setImageDrawable(ContextCompat.getDrawable(mContext,R.drawable.ic_star_border_black_24dp));
            holder.iconImp.setColorFilter(ContextCompat.getColor(mContext,R.color.icon_tint_normal));
        }



        if (!TextUtils.isEmpty(message.getPicture()))
        {

            Glide.with(mContext)
                    .load(message.getPicture())
                    .thumbnail(0.5f)
                    .crossFade()
                    .transform(new CircleTransform(mContext))
                    .diskCacheStrategy(DiskCacheStrategy.ALL)
                    .into(holder.imgProfile);

            holder.imgProfile.setColorFilter(null);
            holder.iconText.setVisibility(View.GONE);

        }else
        {

            holder.imgProfile.setImageResource(R.drawable.bg_circle);
            holder.imgProfile.setColorFilter(message.getColor());
            holder.iconText.setVisibility(View.VISIBLE);

        }



        applyAnimation(holder,position);

    }


    @Override
    public int getItemCount() {
        return messages.size();

    }




    public void toggleSelection(int pos)
    {

        currentSelectedIndex=pos;
        if (selectedItems.get(pos,false))
        {
            selectedItems.delete(pos);
            animationItemsIndex.delete(pos);

        }else
        {
            selectedItems.put(pos,true);
            animationItemsIndex.put(pos,true);
        }

        notifyItemChanged(pos);
    }


    public void clearSelections()
    {
        reverseAllAnimations=true;
        selectedItems.clear();
        notifyDataSetChanged();
    }

    public int getSelectedItemCount()
    {
        return selectedItems.size();
    }


    public List<Integer> getSelectedItems(){
        List<Integer> items=new ArrayList<>(selectedItems.size());
        for (int i=0;i<selectedItems.size();i++)
        {
            items.add(selectedItems.keyAt(i));
        }

        return items;

    }


    public void removeData(int position)
    {
        messages.remove(position);
        currentSelectedIndex=-1;

    }




    public void applyAnimation(MyViewHolder holder,int position)
    {

        if (selectedItems.get(position,false))
        {
            holder.iconFront.setVisibility(View.GONE);

            holder.iconBack.setRotationY(0);

            holder.iconBack.setVisibility(View.VISIBLE);
            holder.iconBack.setAlpha(1);

            if (currentSelectedIndex==position)
            {
                FlipAnimator.flipView(mContext,holder.iconBack,holder.iconFront,true);
                currentSelectedIndex=-1;
            }


        }else
        {

            holder.iconBack.setVisibility(View.GONE);


            holder.iconFront.setRotationY(0);

            holder.iconFront.setVisibility(View.VISIBLE);

            holder.iconFront.setAlpha(1);

            if ((reverseAllAnimations&&animationItemsIndex.get(position,false))||currentSelectedIndex==position)
            {
                FlipAnimator.flipView(mContext,holder.iconBack,holder.iconFront,false);
                currentSelectedIndex=-1;

            }


        }

    }


    public void resetAnimationIndex()
    {
        reverseAllAnimations=false;
        animationItemsIndex.clear();
    }



    public class MyViewHolder extends RecyclerView.ViewHolder
    {

        public TextView from,subject,message,iconText,timestamp;
        public ImageView iconImp,imgProfile;
        public LinearLayout messageContainer;
        public RelativeLayout iconContainer,iconBack,iconFront;



        public MyViewHolder(View view) {
            super(view);

            from=(TextView) view.findViewById(R.id.from);
            subject=(TextView)view.findViewById(R.id.txt_primary);
            message=(TextView)view.findViewById(R.id.txt_secondary);
            iconText=(TextView)view.findViewById(R.id.icon_text);
            timestamp=(TextView)view.findViewById(R.id.timestamp);

            iconImp=(ImageView)view.findViewById(R.id.icon_star);
            imgProfile=(ImageView)view.findViewById(R.id.icon_profile) ;


            messageContainer=(LinearLayout)view.findViewById(R.id.message_container);

            iconContainer=(RelativeLayout)view.findViewById(R.id.icon_container);
            iconBack=(RelativeLayout)view.findViewById(R.id.icon_back);
            iconFront=(RelativeLayout)view.findViewById(R.id.icon_front);



        }
    }


    public  interface MessageAdapterListener
    {
        void onIconClicked( int position);

        void onMessageRowClicked(int position);

        void onRowLongClicked(int position);

    }




}
